
#include "Button.h"

void SDL_Button_Initialize( SDL_Button *button, int x, int y, SDL_Surface *image )
{
    button->dest.x = x;
    button->dest.y = y;

    button->image = image;

    button->src.x = 0;
    button->src.y = 0;
    button->src.w = image->w;
    button->src.h = image->h / 3;

    button->is_rolled_over = false;
    button->is_holding_click = false;
    button->is_clicked = false;
    button->is_released = false;
}

void SDL_Button_Update( SDL_Button *button, SDL_Event event )
{
    switch( event.type )
    {
        case SDL_MOUSEBUTTONDOWN:
        {
            if( button->is_rolled_over == true )
            {
                if( button->is_holding_click == false )
                {
                    button->is_holding_click = true;
                    button->is_clicked = true;
                    button->is_released = false;
                }
                else
                {
                    button->is_clicked = false;
                }
            }
            break;
        }
        case SDL_MOUSEBUTTONUP:
        {
            if( button->is_holding_click == true )
            {
                button->is_holding_click = false;
                button->is_clicked = false;
                button->is_released = true;
            }
            else
            {
                button->is_released = false;
            }
            break;
        }
        case SDL_MOUSEMOTION:
        {
            if(( event.motion.x >= button->dest.x ) &&
               ( event.motion.x <  button->dest.x + button->src.w ) &&
               ( event.motion.y >= button->dest.y ) &&
               ( event.motion.y <  button->dest.y + button->src.h ))
            {
                button->is_rolled_over = true;
            }
            else
            {
                button->is_rolled_over = false;
            }
            break;
        }
    }

    if( button->is_rolled_over == true )
    {
        if( button->is_holding_click == true )
        {
            button->src.y = button->src.h * 2;
        }
        else
        {
            button->src.y = button->src.h;
        }
    }
    else
    {
        button->src.y = 0;
    }
}

void SDL_Button_Display( SDL_Button button, SDL_Surface *screen )
{
    SDL_BlitSurface( button.image, &button.src, screen, &button.dest );
}

Boolean SDL_Button_IsRolledOver( SDL_Button button )
{
    return ( button.is_rolled_over );
}

Boolean SDL_Button_IsClicked( SDL_Button button )
{
    return ( button.is_clicked );
}

Boolean SDL_Button_IsReleased( SDL_Button button )
{
    return ( button.is_released );
}
