
#include "InputBox.h"

void SDL_InputBox_Initialize( SDL_InputBox *input_box, int x, int y, SDL_Surface *image, int max_size_text, int off_text_x, int off_text_y, TTF_Font *font, SDL_Color color_font )
{
    input_box->text = malloc( max_size_text );

    if( input_box->text )
    {
        input_box->dest.x = x;
        input_box->dest.y = y;

        input_box->image = image;

        input_box->src.x = 0;
        input_box->src.y = 0;
        input_box->src.w = image->w;
        input_box->src.h = image->h / 3;

        input_box->text[ 0 ] = '\0';
        input_box->cur_index = 0;
        input_box->max_size_text = max_size_text;

        input_box->is_activ = false;
        input_box->is_full = false;

        input_box->last_letter = 0;
        input_box->cur_letter = 0;
        input_box->last_write = 0;

        input_box->off_text_x = off_text_x;
        input_box->off_text_y = off_text_y;
        input_box->font = font;
        input_box->color_font.r = color_font.r;
        input_box->color_font.b = color_font.b;
        input_box->color_font.g = color_font.g;

        input_box->image_text = TTF_RenderText_Blended( font, "", color_font );
    }
}

char* SDL_InputBox_GetText( SDL_InputBox *input_box )
{
    return input_box->text;
}

void SDL_InputBox_SetText( SDL_InputBox *input_box, const char *text )
{
    int i;

    for( i = 0 ; i < input_box->max_size_text - 1 && text[ i ] ; i++ )
    {
        input_box->text[ i ] = text[ i ];
    }

    input_box->text[ i ] = '\0';
    input_box->cur_index = i;

    if( i == input_box->max_size_text - 1 )
    {
        input_box->is_full = true;
    }

    SDL_FreeSurface( input_box->image_text );
    input_box->image_text = TTF_RenderText_Blended( input_box->font, input_box->text, input_box->color_font );
}

int SDL_InputBox_GetSizeText( SDL_InputBox input_box )
{
    return input_box.cur_index;
}

int SDL_InputBox_GetMaxSizeText( SDL_InputBox input_box )
{
    return input_box.max_size_text - 1;
}

void SDL_InputBox_Update( SDL_InputBox *input_box, SDL_Event event )
{
    switch( event.type )
    {
        case SDL_MOUSEBUTTONDOWN:
        {
            if(( event.button.x >= input_box->dest.x ) &&
               ( event.button.x <  input_box->dest.x + input_box->src.w ) &&
               ( event.button.y >= input_box->dest.y ) &&
               ( event.button.y <  input_box->dest.y + input_box->src.h ))
            {
                input_box->is_activ = true;
            }
            else
            {
                input_box->is_activ = false;
            }

            break;
        }
        case SDL_KEYUP:
        {
            input_box->cur_letter = 0;
            break;
        }
        case SDL_KEYDOWN:
        {
            input_box->cur_letter = 0;

            if( input_box->is_activ == true )
            {
                /* letters */
                if((( event.key.keysym.scancode >= (Uint16)'a' ) && ( event.key.keysym.scancode <= (Uint16)'z' )) ||
                   (( event.key.keysym.scancode >= (Uint16)'A' ) && ( event.key.keysym.scancode <= (Uint16)'Z' )))
                {
                    input_box->cur_letter = event.key.keysym.scancode;
                }
                /* number*/
                else if(( event.key.keysym.scancode >= SDL_SCANCODE_0 ) && ( event.key.keysym.scancode <= SDL_SCANCODE_9 ))
                {
                    input_box->cur_letter = event.key.keysym.sym + '0' - SDL_SCANCODE_0;
                }
                /* remove */
                else if( event.key.keysym.scancode == SDL_SCANCODE_BACKSPACE )
                {
                    input_box->cur_letter = SDL_SCANCODE_BACKSPACE;
                }
                /* Underscore ? */
                else if( event.key.keysym.scancode == SDLK_UNDERSCORE )
                {
                    input_box->cur_letter = '_';
                }
            }
            break;
        }
    }


    if(( input_box->is_activ == true ) && ( input_box->cur_letter > 0 ))
    {
        if(( input_box->last_letter != input_box->cur_letter ) || ( SDL_GetTicks( ) > input_box->last_write + INPUTBOX_WRITE_RATE ))
        {
            if( input_box->cur_letter == SDLK_BACKSPACE )
            {
                if( input_box->cur_index > 0 )
                {
                    input_box->cur_index = input_box->cur_index - 1;
                    input_box->text[ input_box->cur_index ] = '\0';

                    SDL_FreeSurface( input_box->image_text );
                    input_box->image_text = TTF_RenderText_Blended( input_box->font, input_box->text, input_box->color_font );

                    input_box->is_full = false;

                    input_box->last_letter = SDLK_BACKSPACE;
                    input_box->last_write = SDL_GetTicks( );
                }
            }
            else if( input_box->is_full == false )
            {
                input_box->text[ input_box->cur_index ] = input_box->cur_letter;
                input_box->cur_index = input_box->cur_index + 1;
                input_box->text[ input_box->cur_index ] = '\0';

                SDL_FreeSurface( input_box->image_text );
                input_box->image_text = TTF_RenderText_Blended( input_box->font, input_box->text, input_box->color_font );

                if( input_box->cur_index == input_box->max_size_text - 1 )
                {
                    input_box->is_full = true;
                }

                input_box->last_letter = input_box->cur_letter;
                input_box->last_write = SDL_GetTicks( );
            }
        }
    }


    if( input_box->is_activ == true )
    {
        if( input_box->is_full == true )
        {
            input_box->src.y = input_box->src.h * 2;
        }
        else
        {
            input_box->src.y = input_box->src.h;
        }
    }
    else
    {
        input_box->src.y = 0;
    }
}

void SDL_InputBox_Display( SDL_InputBox input_box, SDL_Surface *screen )
{
    SDL_BlitSurface( input_box.image, &input_box.src, screen, &input_box.dest );

    input_box.dest.x = input_box.dest.x + input_box.off_text_x;
    input_box.dest.y = input_box.dest.y + input_box.off_text_y;

    SDL_BlitSurface( input_box.image_text, NULL, screen, &input_box.dest );
}

Boolean SDL_InputBox_IsActiv( SDL_InputBox input_box )
{
    return input_box.is_activ;
}

Boolean SDL_InputBox_IsFull( SDL_InputBox input_box )
{
    return input_box.is_full;
}

void SDL_InputBox_Free( SDL_InputBox input_box )
{
    free( input_box.text );
    SDL_FreeSurface( input_box.image_text );
}
