//If our game need to have the mousewheel?????

#include "mouse.h"

#include <SDL_events.h>
#include <SDL_mouse.h>


#define MOUSE_REPEAT_TICKS 150
#define MOUSE_MOVE_DEAD_ZONE 12
#define TRAIL_NUM_DOTS 4


void MouseInit(Mouse *mouse, Pic *cursor, Pic *trail, const bool hideMouse) //initialisation
{
	memset(mouse, 0, sizeof *mouse);
	mouse->cursor = cursor;
	mouse->trail = trail;
	mouse->ticks = 0;
	mouse->repeatedTicks = 0;
	mouse->hideMouse = cursor != NULL || hideMouse;
	if (hideMouse)
	{
		SDL_ShowCursor(SDL_DISABLE);
	}
}

void MousePrePoll(Mouse *mouse)
{
	memset(mouse->pressedButtons, 0, sizeof mouse->pressedButtons); //clean the array
	memcpy(
            mouse->previousButtons,
            mouse->currentButtons,
            sizeof mouse->previousButtons);
	mouse->wheel = Vec2iZero();
	mouse->previousPos = mouse->currentPos;
	SDL_GetMouseState(&mouse->currentPos.x, &mouse->currentPos.y);
	int scale;
	if (scale == 0) scale = 1;
	mouse->currentPos = Vec2iScaleDiv(mouse->currentPos, scale);
}


void MouseOnButtonDown(Mouse *mouse, Uint8 button)
{
	mouse->currentButtons[button] = 1;
}

void MouseOnButtonUp(Mouse *mouse, Uint8 button)
{
	// The mouse presses who are very fast should be detected
	if (mouse->currentButtons[button] && !mouse->previousButtons[button])
	{
		mouse->pressedButtons[button] = 1;
	}
	mouse->currentButtons[button] = 0;
}

void MouseOnWheel(Mouse *m, const Sint32 x, const Sint32 y)
{
	m->wheel = Vec2iNew(x, y);
}

void MousePostPoll(Mouse *mouse, Uint32 ticks)
{
	int areSameButtonPressed = 1;
	int i;
	for (i = 0; i < 8; i++)
	{
		if (mouse->previousButtons[i] ^ mouse->currentButtons[i])
		{
			areSameButtonPressed = 0;
			break;
		}
	}
	// If more time has elapsed, forget about previous buttons for repeating
	if (mouse->repeatedTicks > MOUSE_REPEAT_TICKS)
	{
		mouse->repeatedTicks -= MOUSE_REPEAT_TICKS;
		for (i = 0; i < 8; i++)
		{
			mouse->pressedButtons[i] |= mouse->currentButtons[i];
		}
		memcpy(
			mouse->pressedButtons,
			mouse->currentButtons,
			sizeof mouse->pressedButtons);
	}
	else
	{
		for (i = 0; i < 8; i++)
		{
			mouse->pressedButtons[i] |=
				mouse->currentButtons[i] && !mouse->previousButtons[i];
		}
	}
	mouse->ticks = ticks;
}

bool MouseHasMoved(const Mouse *m)
{
	return !Vec2iEqual(m->previousPos, m->currentPos);
}

int MouseGetPressed(const Mouse *m)
{
    int i;
	for (i =0 ; i < 8; i++)
	{
		if (m->pressedButtons[i])
		{
			return i;
		}
	}
	return 0;
}

bool MouseIsPressed(const Mouse *m, const int button)
{
	return m->pressedButtons[button];
}

bool MouseIsDown(const Mouse *m, const int button)
{
	return m->currentButtons[button];
}

Vec2i MouseWheel(const Mouse *m)
{
	return m->wheel;
}

// Commands
// Maybe need to open a new file for commands
#define CMD_LEFT            1
#define CMD_RIGHT           2
#define CMD_UP              4
#define CMD_DOWN            8

int MouseGetMove(Mouse *mouse, const Vec2i pos)
{
	int cmd = 0;
	const int dx = abs(mouse->currentPos.x - pos.x);
	const int dy = abs(mouse->currentPos.y - pos.y);
	mouse->mouseMovePos = pos;
	// HACK: player is typically drawn slightly to the right
	mouse->mouseMovePos.x += 8;
	const bool isInDeadZone = dx <= MOUSE_MOVE_DEAD_ZONE && dy <= MOUSE_MOVE_DEAD_ZONE;
	if (!isInDeadZone)
	{
		if (2 * dx > dy)
		{
			if (pos.x < mouse->currentPos.x)		cmd |= CMD_RIGHT;
			else if (pos.x > mouse->currentPos.x)	cmd |= CMD_LEFT;
		}
		if (2 * dy > dx)
		{
			if (pos.y < mouse->currentPos.y)		cmd |= CMD_DOWN;
			else if (pos.y > mouse->currentPos.y)	cmd |= CMD_UP;
		}
	}
	return cmd;
}

void MouseDraw(const Mouse *mouse)
{
	if (mouse->cursor)
	{
		????? //new funtions
	}
	if (mouse->trail)
	{
		const int dx = abs(mouse->currentPos.x - mouse->mouseMovePos.x);
		const int dy = abs(mouse->currentPos.y - mouse->mouseMovePos.y);
		const bool isInDeadZone = (dx <= MOUSE_MOVE_DEAD_ZONE) && (dy <= MOUSE_MOVE_DEAD_ZONE); //
		if (!isInDeadZone)
		{
			// Draw a trail between the mouse move pos and mouse pos
			const Vec2i d = Vec2iMinus(mouse->currentPos, mouse->mouseMovePos);
			int i;
			for (i = 1; i <= TRAIL_NUM_DOTS; i++)
			{
				const Vec2i pos = Vec2iAdd(
					mouse->mouseMovePos,
					Vec2iScaleDiv(Vec2iScale(d, i), TRAIL_NUM_DOTS + 1));
				???? //new functions
			}
		}
	}
}
