#pragma once

#include <SDL_stdinc.h>
#include "vector.h"

#include "pic.h"

// Need to open a new file for pic and color_t
typedef struct
{
	Vec2i size;
	Vec2i offset;
	Uint32 *Data;
} Pic;

typedef struct
{
	uint8_t r;
	uint8_t g;
	uint8_t b;
	uint8_t a;
} color_t;

color_t colorWhite = { 255, 255, 255, 255 };

typedef struct
{
	bool previousButtons[8];
	bool currentButtons[8];
	bool pressedButtons[8];
	Vec2i previousPos;
	Vec2i currentPos;
	Vec2i wheel;
	Pic *cursor;
	Pic *trail;
	Uint32 ticks;
	Uint32 repeatedTicks;
	bool hideMouse;
	Vec2i mouseMovePos;
} Mouse;

void MouseInit(Mouse *mouse, Pic *cursor, Pic *trail, const bool hideMouse);
void MousePrePoll(Mouse *mouse);
void MouseOnButtonDown(Mouse *mouse, Uint8 button);
void MouseOnButtonUp(Mouse *mouse, Uint8 button);
void MouseOnWheel(Mouse *m, const Sint32 x, const Sint32 y);
void MousePostPoll(Mouse *mouse, Uint32 ticks);
bool MouseHasMoved(const Mouse *m);
int MouseGetPressed(const Mouse *m);
bool MouseIsDown(const Mouse *m, const int button);
bool MouseIsPressed(const Mouse *m, const int button);

// Get wheel movement since the last poll
Vec2i MouseWheel(const Mouse *m);

// Get mouse movement from a screen position,also sets whether the mouse trail is drawn, and from where
int MouseGetMove(Mouse *mouse, const Vec2i pos);
void MouseDraw(const Mouse *mouse);
